/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_IMAGE_CORE_IMPLDISPATCH_HPP__
#define YAYI_IMAGE_CORE_IMPLDISPATCH_HPP__

#include <Yayi/core/yayiCommon/common_variant.hpp>



/*!@file
 * Dispatcher for images
 */

namespace yayi 
{


  namespace dispatcher
  {
    template <class I, class T, bool B_WRITE_ONLY> struct s_conversion_policy;
    template <class I, class T, bool B_WRITE_ONLY> struct s_runtime_conversion;
    
    struct s_convertible_runtime_time;
    
    template <class T, class coordinate_type, class allocator_type>
    struct s_conversion_policy<IImage*, Image<T, coordinate_type, allocator_type>&, false >
    {
      typedef s_convertible_runtime_time type;
    };
    
    template <class T, class coordinate_type, class allocator_type>
    struct s_runtime_conversion<IImage*, Image<T, coordinate_type, allocator_type>&, false >
    {
      typedef boost::true_type type;
      
      typedef Image<T, coordinate_type, allocator_type> image_type;
      typedef IImage                                    image_type_interface;
      
      
      typedef typename remove_const<typename remove_reference<image_type>::type>::type    T_without_const_ref;
      typedef typename add_reference<typename add_const<T_without_const_ref>::type>::type T_const_ref;
      
      static bool is_convertible(const image_type_interface* r_) throw()
      {
        if(r_ == 0) 
          return false;
        YAYI_ASSERT(r_->DynamicType() == image_type::Type() && dynamic_cast<const image_type*>(r_) != 0, "type corresponds but not dynamic cast ??");
        return r_->DynamicType() == image_type::Type();
      }
      
      static const image_type& convert(const image_type_interface* r_)
      {
        return dynamic_cast<const image_type&>(*r_);
      }
      static image_type& convert(image_type_interface* r_)
      {
        return dynamic_cast<image_type&>(*r_);
      }
    };
  }
  
} // namespace yayi

#endif /*YAYI_IMAGE_CORE_IMPLDISPATCH_HPP__*/

