/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef IMAGECORE_PYTHON_HPP__
#define IMAGECORE_PYTHON_HPP__

#include <boost/python.hpp>
#include <boost/python/object.hpp>
#include <boost/python/def.hpp>

namespace bpy = boost::python;

#include <Yayi/core/yayiCommon/common_types.hpp>
#include <Yayi/core/yayiImageCore/include/yayiImageCore.hpp>


namespace yayi {

  //! Simple interface iterator wrapper
  class IIteratorWrapper {
    boost::shared_ptr<IIterator> itc, itce;
    IIterator* const itcb;
    //IConstIterator *itc, *itce;
    
  public:
    ~IIteratorWrapper();
    IIteratorWrapper(const IIteratorWrapper& r);
    IIteratorWrapper(IIterator* it_, IIterator* ite_);
        
    IIteratorWrapper& operator++();    
    IIteratorWrapper& operator--();
    
    yaRC setPixel(const IIterator::pixel_type &p) {
      return itc->setPixel(p);
    }
    IIterator::pixel_type getPixel() {
      return itc->getPixel();
    }
    
    IIterator::pixel_type next();
    
    IIterator::pixel_type previous() {
      IIterator::pixel_type p = itc->getPixel();
      this->operator--();
      return p;
    }
    
    IIterator::pixel_type operator*() const {
      IIterator::pixel_type p = itc->getPixel();
      //if(res != yaRC_ok)
      //  throw errors::yaException(res);
      return p;
    }
  
    bool operator==(const IIteratorWrapper& r) const {
      return itc->is_equal(r.itc.get());
    }
    
    bool operator!=(const IIteratorWrapper& r) const {
      return itc->is_different(r.itc.get());
    }
    
    IIterator* ptr() {
      return itc.get();
    }
    IIterator const* ptr() const {
      return itc.get();
    }    
    
  };

}


#endif /* IMAGECORE_PYTHON_HPP__ */
