/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */


#include <Yayi/core/yayiImageCorePython/include/imagecore_python.hpp>

#include <boost/python/class.hpp>
#include <boost/python/manage_new_object.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/lambda/lambda.hpp>

#if 0
namespace boost { namespace python
{
  template <class C>
  struct iterators<>
  {
      typedef typename C::[const_]iterator iterator;
      static iterator begin(C& x);
      static iterator end(C& x);
  };
}}
#endif

namespace yayi {

  IIteratorWrapper::~IIteratorWrapper() {
    //std::cout << "IIteratorWrapper::~IIteratorWrapper : itc = " << (void*)itc.get() << std::endl;
  
  }
  IIteratorWrapper::IIteratorWrapper(const IIteratorWrapper& r) : 
      itc(r.itc), itce(r.itce), itcb(r.itcb)
    {}
  IIteratorWrapper::IIteratorWrapper(IIterator* it_, IIterator* ite_) : itc(it_), itce(ite_), itcb(it_)
  {
    if(!it_ || !ite_)
      throw errors::yaException(yaRC_E_allocation);
  }
  IIteratorWrapper& IIteratorWrapper::operator++() {
    if(itc->is_equal(itce.get())) {
      PyErr_SetObject(PyExc_StopIteration, Py_None);
      boost::python::throw_error_already_set();
    }
    
    yaRC res = itc->next();
    if(res != yaRC_ok)
      throw errors::yaException(res);      

    return *this;
  }
  IIteratorWrapper& IIteratorWrapper::operator--() {
    if(itc->is_equal(itcb)) {
      PyErr_SetObject(PyExc_StopIteration, Py_None);
      boost::python::throw_error_already_set();
    }
    
    yaRC res = itc->previous();
    if(res != yaRC_ok)
      throw errors::yaException(res);      

    return *this;
  }    
  IIterator::pixel_type IIteratorWrapper::next() {
    //std::cout << "XXX next with itc = " << (void*)itc.get() << std::endl;
    IIterator::pixel_type p = itc->getPixel();
    //std::cout << "XXX next p " << variant_field_from_type<yaUINT8>(p) << std::endl;
    this->operator++();
    //std::cout << "XXX next return "  << std::endl;
    return p;
  }


  IIterator::coordinate_type GetPosition(const IIteratorWrapper& it)
  {
    return it.ptr()->GetPosition();
  }
  yaRC SetPosition(IIteratorWrapper& it, const IIterator::coordinate_type& c)
  {
    return it.ptr()->SetPosition(c);
  }
  
  IIterator::pixel_type GetPixelValue(const IIteratorWrapper& it) {
    //std::cout << "XXX GetPixelValue" << std::endl;
    return it.ptr()->getPixel();
  }

  yaRC SetPixelValue(IIteratorWrapper& it, const IIterator::pixel_type& c)
  {
    //std::cout << "YYY GetPixelValue" << std::endl;
    return it.ptr()->setPixel(c);
  }
  
  IIteratorWrapper ItClone(const IIteratorWrapper&it) {return IIteratorWrapper(it);}
  

}



void declare_iterators() {

  using namespace yayi;
  //using namespace boost::lambda;
  
  bpy::class_<IIteratorWrapper>("Iterator", "Iterator over a range on images", bpy::no_init)
    .def("__iter__",    &ItClone, /*bpy::return_value_policy<*/bpy::with_custodian_and_ward_postcall<0,1> /*>*/() )
  
    .def("next",        &IIteratorWrapper::next)
    .def("previous",    &IIteratorWrapper::previous)

    .def(bpy::self == bpy::other<IIteratorWrapper>())
    .def(bpy::self != bpy::other<IIteratorWrapper>())
    //.def("__ne__",      &IIteratorWrapper::operator!=)
    //.def("GetPosition", &IConstIterator::GetPosition)
    .def("GetPosition",   &GetPosition)//boost::lambda::protect(_1.ptr()->GetPosition))
    .def("SetPosition",   &SetPosition)
    .add_property("position",     &GetPosition, &SetPosition)


    .def("SetPixel",   &SetPixelValue)
    .def("GetPixel",   &GetPixelValue)
    .add_property("value",     &GetPixelValue, &SetPixelValue)    

    
  ;
    
  
  
  
}
