/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_RUNTIME_STRUCTURING_ELEMENT_HPP__
#define YAYI_RUNTIME_STRUCTURING_ELEMENT_HPP__

/*!@file
 * This file defines structuring elements that can be set programatically at run time
 *
 * @author Raffi Enficiaud
 */
 
 
namespace yayi {


  struct neighborhood_point_list_tag {};      //!< Tag for SE using a point list as initialization

  /*! A simple structuring element coded by offset
   *
   * @author Raffi Enficiaud
   */
  template <class element_t, class image_t_>
    struct s_vector_se {
    typedef std::vector<element_t> storage_type;
    
    typedef typename storage_type::iterator       iterator;
    typedef typename storage_type::const_iterator const_iterator;
    
  private:
    // We need sometimes to access the image parameters
    const image_t_ *im;
  };
  
  
  
  

  /*! A runtime neighborhood (ie. a neighborhood that can be set at run time)
   *
   * @author Raffi Enficiaud
   */
  template <class image_t_, class se_t_>
    struct s_neighborhood : public INeighborhood 
  {
    
    typedef typename image_t_::coordinate_type coordinate_type;
    
    //! Centers the Neighborhood at a particular point
    yaRC center(const coordinate_type&);
    
    
    
    
    //! Factory for the structuring element
    static createNeighborhood(const image_t_&, const se_t_&);
  };


}





#endif /* YAYI_RUNTIME_STRUCTURING_ELEMENT_HPP__ */
