/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#ifndef YAYI_STRUCTURING_ELEMENT_MAIN_HPP__
#define YAYI_STRUCTURING_ELEMENT_MAIN_HPP__





/*@namespace structuring
 *@brief Structuring element structures and elements
 *
 *
 *@author Raffi Enficiaud
 */



#include "Yayi/core/yayiCommon/common_config.hpp"
#include "Yayi/core/yayiImageCore/include/yayiImageCore.hpp"


#ifdef YAYI_EXPORT_SE_
#define YSE_ MODULE_EXPORT
#else
#define YSE_ MODULE_IMPORT
#endif


namespace yayi {

  namespace se {


    typedef enum e_structuring_element_type {
      e_set_runtime,
      e_set_neighborlist,
      e_set_image,
      e_set_functional,
      e_set_template
    
    } structuring_element_type;


    /*! Interface for structuring elements
     *  @author Raffi Enficiaud
     */
    class IStructuringElement {
    public:
      virtual ~IStructuringElement(){}

      virtual structuring_element_type getType() const = 0;

    };
    
    
    
    /*! Interface for the neighborhood
     *  @author Raffi Enficiaud
     */
    class INeighborhood {
    public:
      virtual ~INeighborhood(){}
    
      //! The iterator type for the neighborhood (generic image iterator interface)
      typedef IImage                                image_type;
      typedef image_type::coordinate_type           coordinate_type;
      typedef IIterator*                            iterator;
      typedef IIterator*                            const_iterator;
      
      //! Centers the Neighborhood at a particular point
      virtual yaRC center(const coordinate_type&) = 0;
      
      
      //! Factory for the structuring element
      static INeighborhood* create(const IImage&, const IStructuringElement&);
  };  
  
  
  
  }
}



#endif