/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */

#include "main.hpp"

#include <Yayi/core/yayiCommon/common_types.hpp>
#include <Yayi/core/yayiCommon/include/common_types_T.hpp>
#include <Yayi/core/yayiCommon/common_variant.hpp>
#include <Yayi/core/yayiCommon/common_pixels.hpp>
#include <Yayi/core/yayiCommon/common_pixels_T.hpp>




class types_and_variant_test 
{
public:

  struct s_dummy{};
  
  void test_types_class()
  {
    BOOST_CHECKPOINT("test_types_class");    
    using namespace yayi;
    
    
    BOOST_CHECKPOINT("test_types_class : tests de serialisation");  
    yayi::type t;
    t.c_type = yayi::type::c_scalar;
    t.s_type = yayi::type::s_ui32;
    
    
    yayi::string_type s = static_cast<yayi::string_type>(t);
    yayi::type tt = yayi::type::Create(s);
    
    BOOST_CHECK_MESSAGE(t == tt, s << " != " << static_cast<yayi::string_type>(tt));   
    
     
    t.c_type = yayi::type::c_coordinate;
    t.s_type = yayi::type::s_ui32;
    
    s = static_cast<yayi::string_type>(t);
    tt = yayi::type::Create(s);

    BOOST_CHECK_MESSAGE(t == tt, s << " != " << static_cast<yayi::string_type>(tt));   
  }
  
  
  void test_types_description()
  {
    using namespace yayi;
    BOOST_CHECKPOINT("test_types_description 1");
    BOOST_CHECK(yayi::type_description::type_desc<yaINT8>::is_pod::value);
    BOOST_CHECK(yayi::type_description::type_desc<yaINT8>::scalar == yayi::type::s_i8);
    BOOST_CHECK(yayi::type_description::type_desc<yaINT8>::name() == std::string("yaINT8"));
    BOOST_CHECK_NO_THROW(yayi::type_description::type_desc<yaINT8>());

    BOOST_CHECK(yayi::type_description::type_desc<yaUINT8>::is_pod::value);
    BOOST_CHECK(yayi::type_description::type_desc<yaUINT8>::scalar == yayi::type::s_ui8);
    BOOST_CHECK(yayi::type_description::type_desc<yaUINT8>::name() == std::string("yaUINT8"));
    BOOST_CHECK_NO_THROW(yayi::type_description::type_desc<yaUINT8>());

    BOOST_CHECK(yayi::type_description::type_desc<yaF_double>::is_pod::value);
    BOOST_CHECK(yayi::type_description::type_desc<yaF_double>::scalar == yayi::type::s_double);
    BOOST_CHECK(yayi::type_description::type_desc<yaF_double>::name() == std::string("yaF_double"));
    BOOST_CHECK_NO_THROW(yayi::type_description::type_desc<yaF_double>());

    typedef std::vector<yaUINT8> vector_type;
    BOOST_CHECK(!yayi::type_description::type_desc<vector_type>::is_pod::value);
    BOOST_CHECK(yayi::type_description::type_desc<vector_type>::scalar == yayi::type::s_ui8);
    BOOST_CHECK(yayi::type_description::type_desc<vector_type>::compound == yayi::type::c_vector);
    BOOST_CHECK(yayi::type_description::type_desc<vector_type>::name() == std::string("vector<yaUINT8>"));


    typedef std::map<yaUINT8, std::vector<yaF_double> > map_type;
    BOOST_CHECK(!yayi::type_description::type_desc<map_type>::is_pod::value);
    BOOST_CHECK(yayi::type_description::type_desc<map_type>::scalar == yayi::type::s_object);
    BOOST_CHECK(yayi::type_description::type_desc<map_type>::compound == yayi::type::c_map);
    BOOST_CHECK_MESSAGE(yayi::type_description::type_desc<map_type>::name() == "map<yaUINT8, vector<yaF_double>>",
      yayi::type_description::type_desc<map_type>::name() << " != " << "map<yaUINT8, vector<yaF_double>>");




#ifndef YAYI_COMPILER_ERROR_FOR_UNSUPPORTED_TYPES__
    typedef s_dummy unsupported_type;
    BOOST_CHECK_THROW(yayi::type_description::type_desc<unsupported_type>(), yayi::errors::yaException);
    BOOST_CHECK(yayi::type_description::type_desc<unsupported_type>::scalar == type::s_undefined);
#endif



  }


  void test_types_transforms()
  {
    using namespace yayi;
    BOOST_CHECKPOINT("test_types_transforms 1");
    variant test_var;
    test_var.element.ui8_value  = 10;
    test_var.element_type.s_type= type_support<yaUINT8>::scalar;
    test_var.element_type.c_type= type_support<yaUINT8>::compound;
    BOOST_CHECK(test_var.element_type.c_type == yayi::type::c_scalar);

    BOOST_CHECKPOINT("test_types_transforms 2");
    variant test_var2 = test_var;
    BOOST_CHECK(test_var.element_type == test_var2.element_type);
    BOOST_CHECK(test_var.element.ui8_value == test_var2.element.ui8_value);

    BOOST_CHECKPOINT("test_types_transforms 3");
    variant test_var3 = (yaUINT8)10;
    
    BOOST_CHECKPOINT("test_types_transforms 4");
    yaF_simple f = test_var3;
    BOOST_CHECK(abs(f - 10) < 1.0E-10);
    
    BOOST_CHECKPOINT("test_types_transforms 5");
    test_var3 = (yaUINT16)100;
    BOOST_CHECKPOINT("test_types_transforms 6");
    f = test_var3;
    BOOST_CHECK(abs(f - 100) < 1.0E-10);



    std::vector<yaF_double> vect;
    for(int i = 0; i < 10; i++)
      vect.push_back(5-i);

    BOOST_CHECKPOINT("test_types_transforms 7");
    variant *test_var4 = new variant(vect);
    variant &test_var4ref = *test_var4;

    BOOST_CHECKPOINT("test_types_transforms 8");
    std::vector<yaINT8> vect2 = test_var4ref;
    for(int i = 0; i < 10; i++)
    {
      BOOST_CHECK(vect[i] == vect2[i]);
    }

    delete test_var4;





#ifndef YAYI_COMPILER_ERROR_FOR_UNSUPPORTED_TYPES__
    BOOST_CHECK_THROW(test_var3 = (int)100);
#endif

  }
  
  void test_types_pixel_types()
  {
    using namespace yayi;
    typedef s_compound_pixel_t<yaUINT32, mpl::int_<5> > pix_5_t;
    BOOST_CHECK(pix_5_t::dimension::value == 5);
    
    pix_5_t v1, v2;
    
    v1[0] = v2[0] = 1;
    v1[1] = v2[1] = 2;    
    v1[2] = v2[2] = 3;
    v1[3] = v2[3] = 3;
    v1[4] = v2[4] = 3;    
    
    
    // Comparisons
    BOOST_CHECK(v1 == v2);
    BOOST_CHECK(!(v1 != v2));
    
    v2[2] = 4;
    BOOST_CHECK(v1 != v2);
    BOOST_CHECK(!(v1 == v2));

    v1 = v2;
    BOOST_CHECK(v1 == v2); 

#ifndef NDEBUG
    BOOST_CHECK_THROW(v2[7], errors::yaException);
#endif

    pix_5_t v3(v2);
    BOOST_CHECK(v3 == v2);
    
   

    // copy constructor
    pix_5_t v4(v2);
    BOOST_CHECK(v4 == v2);
    
    
    s_compound_pixel_t<yaUINT64, mpl::int_<5> > pp(v2);
    
    BOOST_CHECK(pp == v2);
  }

  
  
};

void register_types_and_variant_lifetime_test(test_suite*& test)
{
  boost::shared_ptr<types_and_variant_test> instance( new types_and_variant_test );
  test->add( BOOST_CLASS_TEST_CASE( &types_and_variant_test::test_types_class,        instance ) );
  test->add( BOOST_CLASS_TEST_CASE( &types_and_variant_test::test_types_description,  instance ) );
  test->add( BOOST_CLASS_TEST_CASE( &types_and_variant_test::test_types_transforms,   instance ) );


}

