/*
 *  -:- LICENCE -:- 
 * Copyright Raffi Enficiaud 2007-2009
 * 
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file ../../LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 * 
 *  -:- LICENCE -:- 
 */




#include "main.hpp"
#include "Yayi/core/yayiIO/include/yayi_IO.hpp"
#include "Yayi/core/yayiImageCore/include/yayiImageCore_Impl.hpp"



// test d'appel de fonction, avec test de contenu
void WriteJPGTest() {
  using namespace yayi;
  BOOST_CHECKPOINT("test jpg write");	
  
  typedef Image<yaUINT8> image_type;
  image_type im;
  
  image_type::coordinate_type coord;
  coord[0] = 10;
  coord[1] = 20;
  im.SetSize(coord);
  
  BOOST_REQUIRE_MESSAGE(im.AllocateImage() == yaRC_ok, "im.AllocateImage() error");
  
  for(int i = 0; i < coord[0] * coord[1]; i++) {
    im.pixel(i) = i;
  }
  
  yaRC ret = yayi::IO::writeJPG("test.jpg", &im);
  string_type str = static_cast<string_type>(ret);
  BOOST_CHECK_MESSAGE(ret == yaRC_ok, "writeJPG error " << ret);
}

void ReadJPGTest() {
  using namespace yayi;
  BOOST_CHECKPOINT("test jpg read");	
  
  
  IImage *im = 0;
  yaRC ret = yayi::IO::readJPG("test.jpg", im);
  BOOST_CHECK_MESSAGE(ret == yaRC_ok, "read error " + static_cast<string_type>(ret));
  
  BOOST_REQUIRE_MESSAGE(im->IsAllocated(), "im not allocated ?");

  IImage::coordinate_type coord = im->GetSize();
  //BOOST_CHECK_MESSAGE(ret == yaRC_ok, "cannot get the image size " + static_cast<string_type>(ret));
  BOOST_CHECK_MESSAGE(coord.dimension() == 2, "bad dimension");
  BOOST_CHECK_MESSAGE(coord[0] == 10, "bad dimension");
  BOOST_CHECK_MESSAGE(coord[1] == 20, "bad dimension");

  typedef Image<yaUINT8> image_type;
  image_type *im_t = dynamic_cast<image_type*>(im);
  

  BOOST_CHECK_MESSAGE(im_t != 0, "cast error");
  
  for(int i = 0; i < coord[0] * coord[1]; i++) {
    BOOST_CHECK_MESSAGE(im_t->pixel(i) == i, "pixel bad value");
  }
  
  
  delete im;

}


void WriteJPGTestColor() {
  using namespace yayi;
  BOOST_CHECKPOINT("test jpg write (color)");	
  
  typedef Image<s_compound_pixel_t<yaUINT8,  mpl::int_<3> > > image_type;
  image_type im;
  
  image_type::coordinate_type coord;
  coord[0] = 10;
  coord[1] = 20;
  im.SetSize(coord);
  
  BOOST_REQUIRE_MESSAGE(im.AllocateImage() == yaRC_ok, "im.AllocateImage() error");
  
  for(int i = 0; i < coord[0] * coord[1]; i++) {
    im.pixel(i).a = i;
    im.pixel(i).b = 255-i;
    im.pixel(i).c = 128+i;
  }
  
  yaRC ret = yayi::IO::writeJPG("test_c.jpg", &im);
  string_type str = static_cast<string_type>(ret);
  BOOST_CHECK_MESSAGE(ret == yaRC_ok, "writeJPG error " << ret);
}


void ReadJPGTestColor() {
  using namespace yayi;
  BOOST_CHECKPOINT("test jpg read");	
  
  
  IImage *im = 0;
  yaRC ret = yayi::IO::readJPG("test_c.jpg", im);
  BOOST_CHECK_MESSAGE(ret == yaRC_ok, "read error " + static_cast<string_type>(ret));
  
  BOOST_REQUIRE_MESSAGE(im->IsAllocated(), "im not allocated ?");

  IImage::coordinate_type coord = im->GetSize();
  //BOOST_CHECK_MESSAGE(ret == yaRC_ok, "cannot get the image size " + static_cast<string_type>(ret));
  BOOST_CHECK_MESSAGE(coord.dimension() == 2, "bad dimension");
  BOOST_CHECK_MESSAGE(coord[0] == 10, "bad dimension");
  BOOST_CHECK_MESSAGE(coord[1] == 20, "bad dimension");

  typedef Image<s_compound_pixel_t<yaUINT8,  mpl::int_<3> > > image_type;
  image_type *im_t = dynamic_cast<image_type*>(im);
  

  BOOST_CHECK_MESSAGE(im_t != 0, "cast error");
  
  for(int i = 0; i < coord[0] * coord[1]; i++) {
    BOOST_CHECK_MESSAGE(im_t->pixel(i).a == i, "pixel bad value");
    BOOST_CHECK_MESSAGE(im_t->pixel(i).b == 255-i, "pixel bad value");
    BOOST_CHECK_MESSAGE(im_t->pixel(i).c == 128+i, "pixel bad value");
  }
  
  
  delete im;

}

void register_io_jpeg_test(test_suite*& test) {
  
  test->add( BOOST_TEST_CASE( &WriteJPGTest       ));
  test->add( BOOST_TEST_CASE( &ReadJPGTest        ));
  test->add( BOOST_TEST_CASE( &WriteJPGTestColor  ));
  test->add( BOOST_TEST_CASE( &ReadJPGTestColor   ));
  
}